using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Drawing;

///<summary>
/// Renders a rounded panel on a web form using no images (just CSS). Inherits
/// from the standard Panel which supports addition and rendering of child 
/// controls.
/// 
/// (c) 2007 by Matthew Martin
/// http://www.usualdosage.com
///</summary>

namespace Web2Controls
{
    [DefaultProperty("Text")]
    [ToolboxData("<{0}:RoundedPanel runat=server></{0}:RoundedPanel>")]
    [ToolboxBitmap(typeof(RoundedPanel), "Web2Controls.RoundedPanel.bmp")]
    [Description("Draws a rounded, configurable panel within a panel WebControl.")]
    [ParseChildren(false)]
    [PersistChildren(true)]
    public class RoundedPanel : Panel, System.Web.UI.INamingContainer
    {
        #region [ Properties ]

        [Bindable(true)]
        [Category("Rounded Panel")]
        [DefaultValue("RoundedPanel")]
        [Localizable(true)]
        [Description("The inner text of the rounded panel.")]
        public string Text
        {
            get
            {
                String s = (String)ViewState["Text"];
                return ((s == null) ? String.Empty : s);
            }
            set
            {
                ViewState["Text"] = value;
            }
        }

        private Color _panelTextColor = Color.Black;
        [Bindable(true)]
        [Category("Rounded Panel")]
        [DefaultValue("Black")]
        [Localizable(true)]
        [Description("The color of the text within the rounded panel.")]
        public Color PanelTextColor
        {
            get
            {
                return _panelTextColor;
            }
            set
            {
                _panelTextColor = value;
            }
        }

        private Color _panelInnerBackColor = Color.Gainsboro;
        [Bindable(true)]
        [Category("Rounded Panel")]
        [DefaultValue("Gainsboro")]
        [Localizable(true)]
        [Description("The background color inside of the rounded panel.")]
        public Color PanelInnerBackColor
        {
            get
            {
                return _panelInnerBackColor;
            }
            set
            {
                _panelInnerBackColor = value;
            }
        }

        private Color _panelOuterBackColor = Color.White;
        [Bindable(true)]
        [Category("Rounded Panel")]
        [DefaultValue("White")]
        [Localizable(true)]
        [Description("The background color around the outside of the rounded panel.")]
        public Color PanelOuterBackColor
        {
            get
            {
                return _panelOuterBackColor;
            }
            set
            {
                _panelOuterBackColor = value;
            }
        }

        private Color _panelBorderColor = Color.DarkGray;
        [Bindable(true)]
        [Category("Rounded Panel")]
        [DefaultValue("DarkGray")]
        [Localizable(true)]
        [Description("The border color of the rounded panel.")]
        public Color PanelBorderColor
        {
            get
            {
                return _panelBorderColor;
            }
            set
            {
                _panelBorderColor = value;
            }
        }

        #endregion

        #region [ Overrides ]

        protected override void RenderContents(HtmlTextWriter output)
        {
            string borderColor = ColorTranslator.ToHtml(PanelBorderColor);
            string backColor = ColorTranslator.ToHtml(PanelOuterBackColor);
            string panelColor = ColorTranslator.ToHtml(PanelInnerBackColor);
            string foreColor = ColorTranslator.ToHtml(PanelTextColor);
            string id = ClientID + "_inner";

            StringBuilder sbOut = new StringBuilder();

            // Build the CSS
            sbOut.Append("<style type=\"text/css\">#" + id + " {background:" + backColor + "; width:" + Width + "; height:" + Height + "; margin:0px; padding:0px;}");
            sbOut.Append("#" + id + " .xrounded p {margin:0 10px; letter-spacing:1px;}");
            sbOut.Append("#" + id + " .xrounded p {padding-bottom:0.5em; color:" + foreColor + ";}");
            sbOut.Append("#" + id + " .xrounded {background: transparent; margin:0em;}");
            sbOut.Append("#" + id + " .xrounded em {display:block; width:0; height:0; color:" + backColor +
                "; overflow:hidden; border-top:12px solid " + borderColor + "; border-left:12px dotted transparent; border-right:12px dotted transparent; margin-left:50px;}");
            sbOut.Append("* html .xrounded em {width:24px; height:12px; width:0; height:0;}");
            sbOut.Append("#" + id + " .xrounded span {display:block; width:0; height:0; color:" + borderColor +
                "; overflow:hidden; border-top:10px solid " + panelColor + "; border-left:10px dotted transparent; border-right:10px dotted transparent; margin-left:52px; margin-top:-15px;}");
            sbOut.Append("* html .xrounded span {width:20px; height:10px; width:0; height:0;}");
            sbOut.Append(".xb1, .xb2, .xb3, .xb4, .xb5, .xb6, .xb7 {display:block; overflow:hidden; font-size:0;}");
            sbOut.Append(".xb1, .xb2, .xb3, .xb4, .xb5, .xb6 {height:1px;}");
            sbOut.Append("#" + id + " .xb4, #" + id + " .xb5, #" + id + " .xb6, #" + id +
                " .xb7 {background:#ccc; border-left:1px solid " + borderColor +
                "; border-right:1px solid " + borderColor + ";}");
            sbOut.Append("#" + id + " .xb1 {margin:0 8px; background:" + borderColor + ";}");
            sbOut.Append("#" + id + " .xb2 {margin:0 6px; background:" + borderColor + ";}");
            sbOut.Append("#" + id + " .xb3 {margin:0 4px; background:" + borderColor + ";}");
            sbOut.Append("#" + id + " .xb4 {margin:0 3px; background:" + panelColor + "; border-width:0 5px;}");
            sbOut.Append("#" + id + " .xb5 {margin:0 2px; background:" + panelColor + "; border-width:0 4px;}");
            sbOut.Append("#" + id + " .xb6 {margin:0 2px; background:" + panelColor + "; border-width:0 3px;}");
            sbOut.Append("#" + id + " .xb7 {margin:0 1px; background:" + panelColor + "; border-width:0 3px; height:2px;}");
            sbOut.Append("#" + id + " .xboxcontent {display:block; background:" + panelColor +
                "; border:3px solid " + borderColor + "; border-width:0 3px;}</style>");

            // Build the panel structure
            sbOut.Append("<div id=\"" + id + "\">");
            sbOut.Append("<div class=\"xrounded\" style=\"height: 100%\">");
            sbOut.Append("<b class=\"xb1\"></b><b class=\"xb2\"></b><b class=\"xb3\"></b><b class=\"xb4\"></b><b class=\"xb5\"></b><b class=\"xb6\"></b><b class=\"xb7\"></b>");
            sbOut.Append("<div class=\"xboxcontent\" style=\"height: 100%\">");

            // Write out any text
            if (Text != String.Empty)
                sbOut.Append("<p>" + Text + "</p>");

            output.Write(sbOut.ToString());

            //// Render child controls if necessary, nest in a P tag
            if (HasControls())
            {
                output.Write("<p>");
                this.RenderChildren(output);
                output.Write("</p>");
            }

            // Close all open html tags
            output.Write("</div>");
            output.Write("<b class=\"xb7\"></b><b class=\"xb6\"></b><b class=\"xb5\"></b><b class=\"xb4\"></b><b class=\"xb3\"></b><b class=\"xb2\"></b><b class=\"xb1\"></b>");
            output.Write("</div>");
            output.Write("</div>");
        }

        #endregion
    }
}
